"""..."""

from dataclasses import asdict

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.types import *


class Update(BaseAligo):
    """..."""

    def update_file(self, body: UpdateFileRequest) -> BaseFile:
        """..."""
        response = self._post(V2_FILE_UPDATE, body=body)
        return self._result(response, BaseFile)

    def rename_file(self, body: RenameFileRequest) -> BaseFile:
        """..."""
        return self.update_file(UpdateFileRequest(**asdict(body)))

    # @overload
    # def rename_name(self, body: str, new_name: str, drive_id: str = None) -> BaseFile:
    #     """..."""
    #     pass
    #
    # @overload
    # def rename_name(self, body: BaseFile, new_name: str, drive_id: str = None) -> BaseFile:
    #     """..."""
    #     pass
    #
    # def rename_name(self, body: Union[str, BaseFile], new_name: str, drive_id: str = None) -> BaseFile:
    #     """..."""
    #     if isinstance(body, BaseFile):
    #         body = body.file_id
    #     return self.update_file(UpdateFileRequest(file_id=body, name=new_name, drive_id=drive_id))



"""文件相关"""
from typing import List

from aligo.core import *
from aligo.request import *
from aligo.types import *


class File(Core):
    """..."""

    def get_file_list(self, parent_file_id: str = 'root', drive_id: str = None, body: GetFileListRequest = None,
                      **kwargs) -> List[BaseFile]:
        """获取文件列表"""
        if body is None:
            body = GetFileListRequest(drive_id=drive_id, parent_file_id=parent_file_id, **kwargs)
        result = super(File, self).get_file_list(body)
        return [i for i in result]



"""..."""
from typing import Iterator

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class File(BaseAligo):
    """..."""

    def get_file_list(self, body: GetFileListRequest) -> Iterator[BaseFile]:
        """..."""
        for i in self._list_file(V2_FILE_LIST, body, GetFileListResponse):
            yield i

    # @lru_memoize()  # for cache of get_file_list fun
    # def _cache_get_file_list(self, body: GetFileListRequest) -> GetFileListResponse:
    #     """..."""
    #     response = self._post(V2_FILE_LIST, body=body)
    #     return self._result(response, GetFileListResponse)
    #
    # @overload
    # def get_file_list(self, body: Union[GetFileListRequest, BaseFile], f5: bool = False) -> Iterator[BaseFile]:
    #     """..."""
    #     ...
    #
    # @overload
    # def get_file_list(self, parent_file_id: str = 'root', drive_id: str = None, f5: bool = False) -> Iterator[BaseFile]:
    #     """..."""
    #     ...
    #
    # # @lru_cache # for the generator, repeated acquisition will only get empty
    # def get_file_list(
    #         self,
    #         body: Union[GetFileListRequest, BaseFile] = None,
    #         parent_file_id: str = 'root',
    #         drive_id: str = None,
    #         f5: bool = False
    # ) -> Iterator[BaseFile]:
    #     """
    #     1. If body is missing and list drive root files
    #     2. Usually you only need to provide GetFileListRequest.parent_file_id, default is 'root'
    #     """
    #     # Generate GetFileListRequest
    #     if body is None:
    #         body = GetFileListRequest(parent_file_id=parent_file_id, drive_id=drive_id)
    #
    #     if isinstance(body, BaseFile):
    #         body = GetFileListRequest(drive_id=body.drive_id, parent_file_id=body.file_id)
    #
    #     if body.drive_id is None:
    #         body.drive_id = self.default_drive_id
    #
    #     if f5:
    #         key = self._cache_get_file_list.cache_key(self=self, body=body)
    #         self._cache_get_file_list.cache.delete(key)
    #
    #     resp = self._cache_get_file_list(body=body)
    #     if isinstance(resp, Null):
    #         yield resp
    #         return
    #     for item in resp.items:
    #         yield item
    #     if resp.next_marker != '':
    #         body.marker = resp.next_marker
    #         for it in self.get_file_list(body=body, f5=f5):
    #             yield it



"""..."""

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.types import *


class Drive(BaseAligo):
    """..."""

    def get_drive(self, body: GetDriveRequest) -> BaseDrive:
        """..."""
        response = self._post(V2_DRIVE_GET, body=body)
        return self._result(response, BaseDrive)

    def get_default_drive(self) -> BaseDrive:
        """..."""
        if self._default_drive is None:
            response = self._post(V2_DRIVE_GET_DEFAULT_DRIVE, body=GetDefaultDriveRequest(self._token.user_id))
            self._default_drive = self._result(response, BaseDrive)
        return self._default_drive

    # @lru_memoize()  # all params must be hashable, dataclass must set unsafe_hash=True
    # def _cache_get_drive(self, drive_id: str) -> BaseDrive:
    #     body = GetDriveRequest(drive_id=drive_id)
    #     response = self._post(V2_DRIVE_GET, body=body)
    #     return self._result(response, BaseDrive)
    #
    # def get_drive(self, drive_id: str = None, f5: bool = False) -> BaseDrive:
    #     """
    #     If not provider body, this func is equals get_default_drive
    #     """
    #     if drive_id is None:
    #         drive_id = self.default_drive_id
    #     params = {'drive_id': drive_id}
    #
    #     if f5:
    #         key = self._cache_get_drive.cache_key(self=self, **params)
    #         self._cache_get_drive.cache.delete(key)
    #     return self._cache_get_drive(**params)



"""..."""

from aligo.core import *
from aligo.request import *
from aligo.types import *
from aligo.types.Enum import *


class Update(Core):
    """..."""

    def rename_file(self,
                    name: str,
                    file_id: str,
                    check_name_mode: CheckNameMode = 'refuse',
                    drive_id: str = None) -> BaseFile:
        """重命名文件(夹)"""
        body = RenameFileRequest(
            name=name,
            file_id=file_id,
            check_name_mode=check_name_mode,
            drive_id=drive_id,
        )
        return super(Update, self).rename_file(body)



"""..."""

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *


class Video(BaseAligo):
    """..."""

    def get_video_play_info(self, body: GetVideoPlayInfoRequest) -> GetVideoPlayInfoResponse:
        """..."""
        response = self._post(V2_DATABOX_GET_VIDEO_PLAY_INFO, body=body)
        return self._result(response, GetVideoPlayInfoResponse)

    # @lru_memoize()
    # def _cache_get_video_play_info(self, drive_id, file_id) -> GetVideoPlayInfoResponse:
    #     body = GetVideoPlayInfoRequest(drive_id=drive_id, file_id=file_id)
    #     response = self._post(V2_DATABOX_GET_VIDEO_PLAY_INFO, body=body)
    #     return self._result(response, GetVideoPlayInfoResponse)
    #
    # @overload
    # def get_video_play_info(self, body: GetVideoPlayInfoRequest, f5: bool = False) -> GetVideoPlayInfoResponse:
    #     """..."""
    #     ...
    #
    # @overload
    # def get_video_play_info(self, body: BaseFile, f5: bool = False) -> GetVideoPlayInfoResponse:
    #     """..."""
    #     ...
    #
    # @overload
    # def get_video_play_info(self, file_id: str,
    #                         drive_id: str = None, f5: bool = False) -> GetVideoPlayInfoResponse:
    #     """..."""
    #     ...
    #
    # def get_video_play_info(
    #         self,
    #         body: Union[GetVideoPlayInfoRequest, BaseFile] = None,
    #         file_id: str = None,
    #         drive_id: str = None,
    #         f5: bool = False
    # ) -> GetVideoPlayInfoResponse:
    #     """..."""
    #     if body is None:
    #         body = GetVideoPlayInfoRequest(drive_id=drive_id, file_id=file_id)
    #     if body.drive_id is None:
    #         body.drive_id = self.default_drive_id
    #     params = {'drive_id': body.drive_id, 'file_id': body.file_id}
    #     if f5:
    #         key = self._cache_get_video_play_info.cache_key(self=self, **params)
    #         self._cache_get_video_play_info.cache.delete(key)
    #     return self._cache_get_video_play_info(**params)



"""..."""

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *


class Audio(BaseAligo):
    """..."""

    def get_audio_play_info(self, body: GetAudioPlayInfoRequest) -> GetAudioPlayInfoResponse:
        """获取音频播放信息"""
        response = self._post(V2_DATABOX_GET_AUDIO_PLAY_INFO, body=body)
        return self._result(response, GetAudioPlayInfoResponse)

    # @lru_memoize()
    # def _cache_get_audio_play_info(self, drive_id, file_id) -> GetAudioPlayInfoResponse:
    #     body = GetAudioPlayInfoRequest(drive_id=drive_id, file_id=file_id)
    #     response = self._post(V2_DATABOX_GET_AUDIO_PLAY_INFO, body=body)
    #     return self._result(response, GetAudioPlayInfoResponse)
    #
    # @overload
    # def get_audio_play_info(self, body: GetAudioPlayInfoRequest, f5: bool = False) -> GetAudioPlayInfoResponse:
    #     """..."""
    #     ...
    #
    # @overload
    # def get_audio_play_info(self, body: BaseFile, f5: bool = False) -> GetAudioPlayInfoResponse:
    #     """..."""
    #     ...
    #
    # @overload
    # def get_audio_play_info(self, file_id: str,
    #                         drive_id: str = None, f5: bool = False) -> GetAudioPlayInfoResponse:
    #     """..."""
    #     ...
    #
    # def get_audio_play_info(self, body: Union[GetAudioPlayInfoRequest, BaseFile] = None, file_id: str = None,
    #                         drive_id: str = None,
    #                         f5: bool = False) -> GetAudioPlayInfoResponse:
    #     """..."""
    #     if body is None:
    #         body = GetAudioPlayInfoRequest(file_id=file_id, drive_id=drive_id)
    #     if body.drive_id is None:
    #         body.drive_id = self.default_drive_id
    #     params = {'drive_id': body.drive_id, 'file_id': body.file_id}
    #     if f5:
    #         key = self._cache_get_audio_play_info.cache_key(self=self, **params)
    #         self._cache_get_audio_play_info.cache.delete(key)
    #     return self._cache_get_audio_play_info(**params)
